;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x82be3c28833887a6b2d4a5798ff0211c . "./code.cg2js/cg-acache.fasl")
(#x3b795ba72ab8eb1dafb3df70b0362838 . "./code.cg2js/cg-base.fasl")
(#xfa8aed8e3a4f87c801420640c6c1445d . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x7ed7afa5bb2c85371ab992e09fa932da . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa2e227e97f933a1afd0076a817c8bc98 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x8c4ee2b5e1017c7049c25e4a41841d6e . "./code.cg2js/cg-button.fasl")
(#x7bc6887b4eebbef5cd998cf756cbe5c1 . "./code.cg2js/cg-calendar.fasl")
(#xdc64b77197fd346e3323e9aed6cf1941 . "./code.cg2js/cg-caret.fasl")
(#xdcf2c5557d962d54b15b327b37df8342 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x42ce13d1e959342a01e3f046ed333814 . "./code.cg2js/cg-chart-widget.fasl")
(#xbb82e260a9ff8c2eaecf5d2632ab34ce . "./code.cg2js/cg-check-box.fasl")
(#x6d21e7c16224b44523a13cb74e0e96f9 . "./code.cg2js/cg-choice-list.fasl")
(#xdef4c840c80e250c07ee9566703ae7bc . "./code.cg2js/cg-choose-printer.fasl")
(#x7da6ed9e1ad2e529ed9323390b18089f . "./code.cg2js/cg-class-grid.fasl")
(#x6be5df474b421daf816d67d1bb62ac24 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x2ff25517f1f18840538319abde7bcc21 . "./code.cg2js/cg-class-support.fasl")
(#x27a8fc344ad4817417c21eab233a2bc0 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xfe2d3007984b586880347738d98d0824 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xc61a1c90c328a0f3436e0c11cc253d97 . "./code.cg2js/cg-clipboard.fasl")
(#xad49fdf69c3a13f6e3e0256feb980449 . "./code.cg2js/cg-color-dialog.fasl")
(#x91a1e7f397710dfc41c711a7d13876ce . "./code.cg2js/cg-combo-box.fasl")
(#x5a6e0a9336762a2425ddb6697f600073 . "./code.cg2js/cg-common-control.fasl")
(#xafc9a9dbfb1165230723c801bb88af07 . "./code.cg2js/cg-comtab.fasl")
(#xbef134d4f0ab53a06e43a37f54e8609c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xec6b38880e73c6b10dbd6f3843d169f1 . "./code.cg2js/cg-curve.fasl")
(#xd8a70d150a23bd0e9964fab0da130ad0 . "./code.cg2js/cg-dde-utils.fasl")
(#x646c63a070fe1026ea9f970bb8f004d6 . "./code.cg2js/cg-dev.fasl")
(#x8a570937db87d2dab0242f02f33657d6 . "./code.cg2js/cg-dialog-item.fasl")
(#x751f09ce62e86dee281a7cd2521618e3 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x0327daef00c3e6d053b8aa23ef7b3653 . "./code.cg2js/cg-directory-dialog.fasl")
(#x66106421c0f4c933906514205db3b267 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xcfb1256f15fed3236716d8ef2cc44117 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x18230e29a2da67d5d65547a8c01b4acf . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x0b7cd30e9bc7de32f1222ae4dd61d147 . "./code.cg2js/cg-drawable.fasl")
(#x6bfedbf8e58d94ec6be6e69fea345d3a . "./code.cg2js/cg-dropping-outline.fasl")
(#xdfbcc782aca80d5c3db08ce801f77e5e . "./code.cg2js/cg-edit-in-place.fasl")
(#x00003654218182a74c8e06357f160162 . "./code.cg2js/cg-editable-text.fasl")
(#x0cf5b8583d884160eab1e3ce5ecb047f . "./code.cg2js/cg-file-dialog.fasl")
(#xde1f13a0fd9a0532ba7e50ef4a920ff9 . "./code.cg2js/cg-fill-texture.fasl")
(#x576ce4371e0997f21d8397b8324163f6 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xb1f3a5013a6be703ca71228f7ea81938 . "./code.cg2js/cg-font-dialog.fasl")
(#xc2722f852d9bb73f396535167d7b1832 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xfb285702c2bf9fc6a4314bc82a60e202 . "./code.cg2js/cg-get-pixmap.fasl")
(#x6c3f6588e5f2ac7c9d042db4c793730e . "./code.cg2js/cg-get-position.fasl")
(#x2044a42e49bbe4c39a0c0abc1a32c85d . "./code.cg2js/cg-graphics-context.fasl")
(#xeb6a1bf578dca53a2f3b28e59352e7d1 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x2a10589312bfa0d750849df394a69250 . "./code.cg2js/cg-grid-widget.fasl")
(#x01d16578088dbb44f5b22e99ecc75e3c . "./code.cg2js/cg-group-box.fasl")
(#x655f9a037f3e3af8530ad7e512271877 . "./code.cg2js/cg-header-control.fasl")
(#x472b5f679a6b54bf31238b4c14180277 . "./code.cg2js/cg-hotspot.fasl")
(#x5daa4785664b9a129a5d894fef309a07 . "./code.cg2js/cg-html-dialog.fasl")
(#x18f1ac681ef0516e91c5e035081f8c2d . "./code.cg2js/cg-html-widget.fasl")
(#x41a9f989f3d08a5a9df988513800d1d9 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xab329131df0fe6695d96272822f154cb . "./code.cg2js/cg-icon.fasl")
(#x89b9f5129b177ca8a7d31993c724f61b . "./code.cg2js/cg-ie.fasl")
(#x5fbcacdd67b765fbcd86137b03b344a4 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0b454afc85032bfe2948de521bcf65d7 . "./code.cg2js/cg-intersect.fasl")
(#x5001e3ee790f449ded3635fd5f2db274 . "./code.cg2js/cg-item-list.fasl")
(#xea0f520651ba4dd2cf9b7d13b26972ac . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xfee6bf455dd94f61db0d775eb6bc9bec . "./code.cg2js/cg-lamp.fasl")
(#x61f96e5c66ad92f9d92ee48fd9cc20ca . "./code.cg2js/cg-layout.fasl")
(#x15eb4ca32b3f0c7b061366c3a8d8f985 . "./code.cg2js/cg-lettered-menu.fasl")
(#x6d2f7ea3f96fefcb16ac09d675b5793c . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9203d28b348b97ddf98c27f7a95b3803 . "./code.cg2js/cg-lisp-text.fasl")
(#x612fe861d63b223b1cf98b444e2bd39b . "./code.cg2js/cg-lisp-widget.fasl")
(#x8c6cfe7a78ce043c39bd41f29913c59b . "./code.cg2js/cg-list-view.fasl")
(#xea52e770fbbc746f3480237922365aa3 . "./code.cg2js/cg-mci.fasl")
(#x36fc0d8984e5dc6df350d77ea0eecb3c . "./code.cg2js/cg-menu-tooltip.fasl")
(#x747e588cba320b8982c326013fdd9573 . "./code.cg2js/cg-menu.fasl")
(#xf27715ce0d3d3f852708ddeda25a9e75 . "./code.cg2js/cg-message-dialog.fasl")
(#x4d8afb4293664e4c4113f45ab8e3ca53 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x545e69921388bcaf37f1f396ed60758f . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x63b0258a13eb390dfb4e9cc9f266d6bd . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x85179734a2567230d0eda30a1db38aa3 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x231623d5ebd67e092f438e948bad0772 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xb8d284bcef21432fecf15e595d7e555b . "./code.cg2js/cg-nodes.fasl")
(#x01300de43b7e61888e96f6000905e9bb . "./code.cg2js/cg-object-editor-layout.fasl")
(#xbeb67458af26c0654a3567baf182c2fd . "./code.cg2js/cg-object-editor.fasl")
(#xb8187304eac404e1d45bec6f32215138 . "./code.cg2js/cg-ocx.fasl")
(#xff3123087c829e095701782ef221b7fd . "./code.cg2js/cg-os-widget.fasl")
(#x184d7dd7fa278cb32c2da8ce27c6df74 . "./code.cg2js/cg-os-window.fasl")
(#xd2b11c6ed77d7176ab94ed3eb48f7e11 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xd72a874ccef05983c1be092e76ec25bf . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1885255dcf64bbe10092e057844ead2a . "./code.cg2js/cg-outline.fasl")
(#xa62590944aa33a7abf8b0ac7660a15ef . "./code.cg2js/cg-palette.fasl")
(#x460270e537a736c3fd32ef6c7de05f1a . "./code.cg2js/cg-paren-matching.fasl")
(#xf15897a65d71e31fb93b3f42ba5b0c41 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x4196753a5b7bf00adb5f04a80f6dd90b . "./code.cg2js/cg-picture-widget.fasl")
(#x4c06408f52450079857e548efac2030c . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x5814a0e0c69f815719245291c87d0267 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x478f331a04a48e5955d746e91a03280a . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x1504106878a2b097fc004e31df02942a . "./code.cg2js/cg-pixmap-widget.fasl")
(#x09e80007a4b1cd162550e7079bb70942 . "./code.cg2js/cg-pixmap.fasl")
(#x7c1238c0d5f7b1e5846e2f7df77d9df6 . "./code.cg2js/cg-player.fasl")
(#x8bace8f912c65deb1ae5f0b5ba918f61 . "./code.cg2js/cg-plot-widget.fasl")
(#xed4ec8da0cfc296ef7707b4954a94186 . "./code.cg2js/cg-printing.fasl")
(#xe284878b5882e597d31b152a1220c6c0 . "./code.cg2js/cg-progress-indicator.fasl")
(#xdf809ebadef7a98a1e014236cd1424d0 . "./code.cg2js/cg-project-window.fasl")
(#x45658bb5f90d1c5c9dc0ae4c9a8e6e9d . "./code.cg2js/cg-property.fasl")
(#x00d07dd4585a13b898b401dc4e6eebb8 . "./code.cg2js/cg-radio-button.fasl")
(#x9bdf36d1cbfe95ef670fd492556c21b6 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xdf8717dbd7919add0369d1bac1d87db2 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x45d76c0452fdd93f43f1adeb275524df . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x2cad459f12eb9668defeb8e57c235c4b . "./code.cg2js/cg-rich-edit.fasl")
(#x9330a2ca44593b0fa9df1bbe892cd360 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xd1ad8fc66acf89201502ec040a40bb80 . "./code.cg2js/cg-scaling-stream.fasl")
(#x6f95a3583e6d0cca358ed6d95c20a008 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x2cf52392aa558a7fc8f1dc8e3230d3be . "./code.cg2js/cg-scroll-bar.fasl")
(#x61b94d05ea0a98c7b5a2a48bfde83120 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xe18190da46ba7e50f6505af0e09a0209 . "./code.cg2js/cg-selected-object.fasl")
(#xb4e98162ad68c340ed022f25cd08d582 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x611c3ffb86147d82d06f19a8cc4b9b87 . "./code.cg2js/cg-split-bar.fasl")
(#xcaee62dff6957befa2587b754f456ccf . "./code.cg2js/cg-static-text.fasl")
(#xe2ae3ddb6d17157eee11cb9df49df668 . "./code.cg2js/cg-status-bar.fasl")
(#x7a5b2b207753ae6102a78169a80abac9 . "./code.cg2js/cg-string-dialog.fasl")
(#x5feab3904580e3dd1f9da9bced74fe05 . "./code.cg2js/cg-tab-control.fasl")
(#xce4c9eebb10913e9b06ac06903a28a92 . "./code.cg2js/cg-template-string.fasl")
(#x9ec2e03819c53bd9524747ec190004ac . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xee651e1936b35e3d92c4604a232b10cb . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x7e5e33dd1d4e9e887bbaae1109a4e9f0 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x52604bd1e12a64ccd3a60d75ca47ce09 . "./code.cg2js/cg-text-or-combo.fasl")
(#x973feca328cb0dc7ee06d8d5c1cdcfcc . "./code.cg2js/cg-text-widget.fasl")
(#x908c7c5d557ce9a5af1f5336edf9e4aa . "./code.cg2js/cg-timer.fasl")
(#xf1d2b4031f7c7173650ae2ff60c7674f . "./code.cg2js/cg-toggling-widget.fasl")
(#x1cb4ed5a3813c2f18052c8043e6b6a1a . "./code.cg2js/cg-toolbar.fasl")
(#x8925ff90f47443d0ae9ed402d6c762b2 . "./code.cg2js/cg-tooltip.fasl")
(#xb17a01a9961d72aa30322f0c52238a73 . "./code.cg2js/cg-touch.fasl")
(#x308b5c121bd148ef95f376fdf772ffbc . "./code.cg2js/cg-trackbar.fasl")
(#xe2bfcaddd258324f7d5e5cf7bc6ee194 . "./code.cg2js/cg-tray.fasl")
(#x771633cbf1552c2a2b1978201a66f208 . "./code.cg2js/cg-up-down-control.fasl")
(#x337b05b4c480f322aee4bfaf128b8df6 . "./code.cg2js/cg-utility-dialog.fasl")
(#x343a41dd88cd91f29129a6bb6a2e3872 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x923088fe56e092d189dba199ff519158 . "./code.cg2js/cg-web-browser.fasl")
(#x66425c54941d02ccef625438243d45b3 . "./code.cg2js/cg-wrap-string.fasl")
(#xdc2e5ca490745a2a2dfad448be6b02d5 . "./code.cg2js/cg-yes-no-list.fasl")
(#x70e2728db9888950e6d6760f00c66de9 . "./code.cg2js/cg.fasl")
(#x8a2a4ede8a03609e68ced03382c14bdf . "./code.cg2js/cg-yes-no-string.fasl")
(#xfa9046494e959f97ae1c1bcd68bbd8c4 . "./code.idejs/ide-acache.fasl")
(#x64260adcae3d31b04f3c4b6adaf78e65 . "./code.idejs/ide-apropos.fasl")
(#xd503362649398527a2e14f74b1962361 . "./code.idejs/ide-base.fasl")
(#x6e6e917a335952f1cd0c425f65251689 . "./code.idejs/ide-class-browser.fasl")
(#xf8342d5b9141dbefaea3c27e07fd2164 . "./code.idejs/ide-class-grapher.fasl")
(#xeac63938baaf0f5709c1be7511d72314 . "./code.idejs/ide-clipboard.fasl")
(#x2d32ffbe807e0974c1d562c6b54f5042 . "./code.idejs/ide-dev.fasl")
(#x996e564b265d4b5a4d002a1eb53be2a9 . "./code.idejs/ide-editor.fasl")
(#xc6324484c439d7ddd9897fca7ced7372 . "./code.idejs/ide-find-definitions.fasl")
(#x90585fff80f4be54c930844453db159c . "./code.idejs/ide-find-in-files.fasl")
(#xd9960ea6a95d8c706c8e7f74c5ab7ffd . "./code.idejs/ide-form.fasl")
(#x1c285d9c9057aee37d25559e53631b90 . "./code.idejs/ide-grapher.fasl")
(#xc612f7bf3e37734b71efa7c40cc95d80 . "./code.idejs/ide-inspector.fasl")
(#xd784278487bf3f8b7bed23b4b67af082 . "./code.idejs/ide-intro.fasl")
(#xcf9d5fb4a6f7c690df5dd9d8a701b647 . "./code.idejs/ide-keyshort.fasl")
(#x4578d31b983efb84c98a38458cfcff60 . "./code.idejs/ide-navigator.fasl")
(#xc05e1c2f4a2fee285ad4636f06e46de8 . "./code.idejs/ide-package-list.fasl")
(#xff6e23d0e73f0e83fb694f1f67fa558e . "./code.idejs/ide-patch-dialog.fasl")
(#xf3c2a1df36ae9379fb7b5f7efefc7c35 . "./code.idejs/ide-process-browser.fasl")
(#xf17b0e3e10ec41dee67871b4ee7f1cb5 . "./code.idejs/ide-project.fasl")
(#xa5863c9050a93e48ee26d046b0b3bbdc . "./code.idejs/ide-trace.fasl")
(#x89bf37f5dbe0fe7d10320b238dd1b033 . "./code.idejs/ide-window-list.fasl")
(#x2fc63cd111a2b991f493833a9d6b7d9e . "./code.idejs/ide.fasl")
(#x3d70f7088b7ddd139f75c06fecd8ac58 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
